/*
  Having just setup a new network in a big office, I wanted a way to test not only that
  the wiring was good, but that I was connected to the switch and could see the internet.
  There are very nice / very expensive tools on the market to do this, but I figured
  I could whip this up myself for less than $25 in parts and get a bunch of experience
  with the Arduino Ethernet modules. 

  Inspired by one of these tools, we have 4 status lights for determining the following: 
    1. Is the link up - are we connected to a switch / hub? 
    2. Can we get a DHCP address? If we can't let's assign a Static. 
    3. Can we perform a DNS lookup? 
    4. Can we connect to a web page? 

  All 4 items will result in a status of GREEN if we can, and RED if we can't.
  So we'll be able to track down the network problem without having to plug in a laptop or PC. 

 Circuit:
 * Ethernet shield attached to pins 10, 11, 12, 13
 * +V Green LED from Ethernet RJ45 jack wired to A0
    There probably is a better way to determine link status with software, but I'm a simple hardware hacker. :D

   Kristopher Marciniak 2016
*/
#include <SPI.h>
#include <UIPEthernet.h> // https://github.com/ntruchsess/arduino_uip
#include <Dns.h>
#define vthresh 30 // Kris's hardware hack. I saw > 300 when the green light was active and < 10 when it wasn't.

byte mac[] = {
  0x00, 0xC0, 0x17, 0x4B, 0x44, 0x4D // A familiar MAC address ;)
};
unsigned int link = 0;

IPAddress ip(10, 0, 1, 29);
String myIP = "10.0.1.29";
IPAddress gateway(10, 0, 1, 1);
IPAddress subnet(255, 255, 255, 0);
IPAddress dnServer(8, 8, 4, 4); // Google DNS
char server[] = "www.google.com";    // Name address for Google (using DNS)
IPAddress timeServer; // Test something besides Google is probably a good idea

String stringIP = "";
boolean DHCP = false;

// Initialize the Ethernet client library
// with the IP address and port of the server
// that you want to connect to (port 80 is default for HTTP):
EthernetClient client;

void setup() {
  pinMode(2, OUTPUT); //LINK GREEN PAIR WITH 6
  pinMode(3, OUTPUT); //DHCP GREEN PAIR WITH 7
  pinMode(4, OUTPUT); //DNS GREEN PAIR WITH 8
  pinMode(5, OUTPUT); //CONNECT GREEN PAIR WITH 9
  pinMode(6, OUTPUT); //LINK RED PAIR WITH 2
  pinMode(7, OUTPUT); //DHCP RED PAIR WITH 3
  pinMode(8, OUTPUT); //DNS RED PAIR WITH 4
  pinMode(9, OUTPUT); //CONNECT RED PAIR WITH 5
  pinMode(A0, INPUT); //+V Green LED from Ethernet RJ45 jack wired to A0

  
  // Open serial communications and wait for port to open:
  Serial.begin(9600);
  // this check is only needed on the Leonardo:
  while (!Serial) {
    ; // wait for serial port to connect. Needed for Leonardo only
  }

  link = analogRead(A0);
  if (link > vthresh) {digitalWrite(2, HIGH); digitalWrite(6, LOW); Serial.println("Ethernet link is active!");}
  else {digitalWrite(2, LOW); digitalWrite(6, HIGH);  Serial.println("Ethernet link is DOWN!");}

  // start the Ethernet connection:
  if (Ethernet.begin(mac) == 0) {
    digitalWrite(3, LOW); digitalWrite(7, HIGH);
    Serial.println("Failed to configure Ethernet using DHCP");
    // no point in carrying on, so do nothing forevermore:
    delay(500); // This will blink DHCP red for this long.
    Ethernet.begin(mac, ip);  
  }
  // print your local IP address:
  if (link > vthresh) {digitalWrite(3, HIGH); digitalWrite(7, LOW);}
  Serial.print("My IP address: ");
  for (byte thisByte = 0; thisByte < 4; thisByte++) {
    // print the value of each byte of the IP address:
    Serial.print(Ethernet.localIP()[thisByte], DEC);
    Serial.print(".");
  }
  Serial.println();
  ipToString(Ethernet.localIP());
  if (stringIP == myIP) {DHCP = false; Serial.println("Using STATIC IP"); }
  else {DHCP = true; Serial.println("Using DHCP IP");}

  DNSClient dns;
  dns.begin(Ethernet.dnsServerIP());
  // Kris says - This doesn't have to be a time server. Put what you want here for DNS lookup.
  // It seemed reasonable that you may want to talk to a time server in the future AND
  // why not check for DNS of something other than Google (which we specified as our DNS server).
  
  if(dns.getHostByName("0.pool.ntp.org",timeServer) == 1) {
    Serial.print(F("ntp = "));
    Serial.println(timeServer);
    digitalWrite(4, HIGH); digitalWrite(8, LOW);
  }
  else {Serial.print(F("dns lookup failed")); digitalWrite(4, LOW); digitalWrite(8, HIGH);}

  if (client.connect(server, 80)) {
    Serial.println("Connected");
    // Make a HTTP request:
    client.println("GET /search?q=arduino HTTP/1.1");
    client.println("Host: www.google.com");
    client.println("Connection: close");
    client.println();
    digitalWrite(5, HIGH); digitalWrite(9, LOW);
  } else {
    // if you didn't get a connection to the server:
    Serial.println("Connection failed");
    digitalWrite(5, LOW); digitalWrite(9, HIGH);
  }


  
}

String ipToString(IPAddress ip){
  stringIP="";
  for (int i=0; i<4; i++)
    stringIP += i  ? "." + String(ip[i]) : String(ip[i]);
  return stringIP;
}

void loop() {
  link = analogRead(A0);
  if (link > vthresh) {digitalWrite(2, HIGH); digitalWrite(6, LOW);}
  else {digitalWrite(2, LOW); digitalWrite(6, HIGH);}

  if (!client.connected()) {
    digitalWrite(5, LOW); digitalWrite(9, HIGH);
    client.stop();}
  
  delay(150); // FANCY blink!
  if (DHCP == false) {digitalWrite(3, LOW); digitalWrite(7, HIGH);}
  if (link <= vthresh) {digitalWrite(6, LOW); }
  delay(150);
  if (link <= vthresh) {digitalWrite(3, LOW); digitalWrite(7, HIGH); digitalWrite(6, HIGH); }
  else {digitalWrite(3, HIGH); digitalWrite(7, LOW);}

}


